/*
//       Elvees Performance Platform
//            epp_ecl API
//
//  Contents:
//
//  Author:
//      Nikolay Degtyarenko.
//
//  Copyright 2023 RnD Center "ELVEES", JSC
*/

#pragma once

#include <eppdefs.h>
#include <elcorecl/elcorecl.h>


void ecl_wait_events(ecl_event* event, ecl_uint num);

void ecl_wait_event(ecl_event event);


class EclBuffer
{
public :
    static const size_t m_alignment = 64;
    EclBuffer();
    EclBuffer(ecl_context context, ecl_command_queue command_queue, size_t size);
    EclBuffer(ecl_context context, ecl_command_queue command_queue, void* ptr, size_t size);
    EclBuffer(ecl_context context, ecl_command_queue command_queue, int dmabuf_fd, size_t size);
    ~EclBuffer();
    void Init(ecl_context context, ecl_command_queue command_queue, size_t size);
    void Init(ecl_context context, ecl_command_queue command_queue, void* ptr, size_t size);
    void Init(ecl_context context, ecl_command_queue command_queue, int dmabuf_fd, size_t size);
    void MapRead();
    void MapWrite();
    void MapReadWrite();
    void Unmap();
    void Close();
    void Read(void* dst, size_t size);
    void Write(const void* src, size_t size);
    bool IsInit() const { return m_is_init; }
    void* GetUserPtr() const { return m_ptr_u; }
    size_t GetUserSize() const { return m_size_u; }
    void* GetAlignedPtr() const { return m_ptr_a; }
    size_t GetAlignedSize() const { return m_size_a; }
    ecl_mem GetMem() const { return m_mem; }
private :
    ecl_mem m_mem;
    bool m_allocated;
    void *m_ptr_u;
    size_t m_size_u;
    void *m_ptr_a;
    size_t m_size_a;
    ecl_command_queue m_command_queue;
    bool m_is_init;
    void SetNotInit();
};


class EclKernel
{
public :
    EclKernel();
    EclKernel(ecl_program program, const char* name);
    ~EclKernel();
    void Init(ecl_program program, const char* name);
    void Close();
    template <class P> void SetArg(ecl_uint num, P arg);
    void SetArg(ecl_uint num, ecl_mem arg, bool non_cacheable = false);
    template <class P> void AddArg(P arg)
    {
        SetArg<P>(m_num_args, arg);
        m_num_args ++;
    }
    void AddArg(ecl_mem arg, bool non_cacheable = false)
    {
        SetArg(m_num_args, arg, non_cacheable);
        m_num_args ++;
    }
    ecl_event Execute(ecl_command_queue command_queue, bool wait = true);
    bool IsInit() { return m_is_init; };
private :
    ecl_kernel m_kernel;
    size_t m_num_args;
    bool m_is_init;
    void SetNotInit();
};


class EppEcl
{
public :
    EppEcl();
    EppEcl(const char* elf_path, ecl_uint dev_num);
    ~EppEcl();
    void Init(const char* elf_path, ecl_uint dev_num);
    void Close();
    bool IsInit() { return m_is_init; };
    ecl_device_id GetDeviceId() { return m_dev_id; }
    ecl_context GetContext() { return m_context; }
    ecl_program GetProgram() { return m_program; }
    ecl_command_queue GetCommandQueue() { return m_command_queue; }
    void nn_MulCAddC_u8f32_C3(const EclBuffer& src, const float v_mul[3], const float v_add[3], EclBuffer& dst, size_t height, size_t width);
    void nn_MulCAddC_u8f32_C3P3(const EclBuffer& src, const float v_mul[3], const float v_add[3], EclBuffer& dst, size_t height, size_t width);
    void nn_Resize_u8_C3(const EclBuffer& src, size_t sHeight, size_t sWidth, EclBuffer& dst, size_t dHeight, size_t dWidth, epp_InterpolationMode im);
    void i_YPbPr444_RGB_BT601_u8_P3C3(const EclBuffer& src, const size_t srcOffs[3], const int srcStep[3], EclBuffer& dst, int dstStep, size_t width, size_t height);
    void i_YPbPr422_RGB_BT601_u8_P3C3(const EclBuffer& src, const size_t srcOffs[3], const int srcStep[3], EclBuffer& dst, int dstStep, size_t width, size_t height);
    void i_YPbPr420_RGB_BT601_u8_P3C3(const EclBuffer& src, const size_t srcOffs[3], const int srcStep[3], EclBuffer& dst, int dstStep, size_t width, size_t height);
    void i_YPbPrVUYA_RGBA_BT601_u8_AC4(const EclBuffer& src, int srcStep, EclBuffer& dst, int dstStep, size_t width, size_t height);
    void i_YPbPrYUYV_RGB_BT601_u8_C2C3(const EclBuffer& src, int srcStep, EclBuffer& dst, int dstStep, size_t width, size_t height);
    void i_YPbPrNV12_RGB_BT601_u8_P2C3(const EclBuffer& src, const size_t srcOffs[2], const int srcStep[2], EclBuffer& dst, int dstStep, size_t width, size_t height);
    void i_YCbCr444_RGBs_BT601_u8_P3C3(const EclBuffer& src, const size_t srcOffs[3], const int srcStep[3], EclBuffer& dst, int dstStep, size_t width, size_t height);
    void i_YCbCr422_RGBs_BT601_u8_P3C3(const EclBuffer& src, const size_t srcOffs[3], const int srcStep[3], EclBuffer& dst, int dstStep, size_t width, size_t height);
    void i_YCbCr420_RGBs_BT601_u8_P3C3(const EclBuffer& src, const size_t srcOffs[3], const int srcStep[3], EclBuffer& dst, int dstStep, size_t width, size_t height);
    void i_YCbCrVUYA_RGBAs_BT601_u8_AC4(const EclBuffer& src, int srcStep, EclBuffer& dst, int dstStep, size_t width, size_t height);
    void i_YCbCrYUYV_RGBs_BT601_u8_C2C3(const EclBuffer& src, int srcStep, EclBuffer& dst, int dstStep, size_t width, size_t height);
    void i_YCbCrNV12_RGBs_BT601_u8_P2C3(const EclBuffer& src, const size_t srcOffs[2], const int srcStep[2], EclBuffer& dst, int dstStep, size_t width, size_t height);
    void i_YPbPr444_RGB_BT2020_u8_P3C3(const EclBuffer& src, const size_t srcOffs[3], const int srcStep[3], EclBuffer& dst, int dstStep, size_t width, size_t height);
    void i_YPbPr422_RGB_BT2020_u8_P3C3(const EclBuffer& src, const size_t srcOffs[3], const int srcStep[3], EclBuffer& dst, int dstStep, size_t width, size_t height);
    void i_YPbPr420_RGB_BT2020_u8_P3C3(const EclBuffer& src, const size_t srcOffs[3], const int srcStep[3], EclBuffer& dst, int dstStep, size_t width, size_t height);
    void i_YPbPrVUYA_RGBA_BT2020_u8_AC4(const EclBuffer& src, int srcStep, EclBuffer& dst, int dstStep, size_t width, size_t height);
    void i_YPbPrYUYV_RGB_BT2020_u8_C2C3(const EclBuffer& src, int srcStep, EclBuffer& dst, int dstStep, size_t width, size_t height);
    void i_YPbPrNV12_RGB_BT2020_u8_P2C3(const EclBuffer& src, const size_t srcOffs[2], const int srcStep[2], EclBuffer& dst, int dstStep, size_t width, size_t height);
    void i_Duplicate_u8_C1C3(const EclBuffer& src, int srcStep, EclBuffer& dst, int dstStep, size_t width, size_t height);
private :
    ecl_platform_id m_platform_id;
    ecl_device_id m_dev_id;
    ecl_context m_context;
    ecl_program m_program;
    ecl_command_queue m_command_queue;
    EclKernel m_kernel_core_Init;
    EclKernel m_kernel_nn_MulCAddC_u8f32_C3;
    EclKernel m_kernel_nn_MulCAddC_u8f32_C3P3;
    EclKernel m_kernel_nn_Resize_u8_C3;
    EclKernel m_kernel_i_YPbPr444_RGB_BT601_u8_P3C3;
    EclKernel m_kernel_i_YPbPr422_RGB_BT601_u8_P3C3;
    EclKernel m_kernel_i_YPbPr420_RGB_BT601_u8_P3C3;
    EclKernel m_kernel_i_YPbPrVUYA_RGBA_BT601_u8_AC4;
    EclKernel m_kernel_i_YPbPrYUYV_RGB_BT601_u8_C2C3;
    EclKernel m_kernel_i_YPbPrNV12_RGB_BT601_u8_P2C3;
    EclKernel m_kernel_i_YCbCr444_RGBs_BT601_u8_P3C3;
    EclKernel m_kernel_i_YCbCr422_RGBs_BT601_u8_P3C3;
    EclKernel m_kernel_i_YCbCr420_RGBs_BT601_u8_P3C3;
    EclKernel m_kernel_i_YCbCrVUYA_RGBAs_BT601_u8_AC4;
    EclKernel m_kernel_i_YCbCrYUYV_RGBs_BT601_u8_C2C3;
    EclKernel m_kernel_i_YCbCrNV12_RGBs_BT601_u8_P2C3;
    EclKernel m_kernel_i_YPbPr444_RGB_BT2020_u8_P3C3;
    EclKernel m_kernel_i_YPbPr422_RGB_BT2020_u8_P3C3;
    EclKernel m_kernel_i_YPbPr420_RGB_BT2020_u8_P3C3;
    EclKernel m_kernel_i_YPbPrVUYA_RGBA_BT2020_u8_AC4;
    EclKernel m_kernel_i_YPbPrYUYV_RGB_BT2020_u8_C2C3;
    EclKernel m_kernel_i_YPbPrNV12_RGB_BT2020_u8_P2C3;
    EclKernel m_kernel_i_Duplicate_u8_C1C3;
    bool m_is_init;
    void SetNotInit();
    void core_Init(epp_ExecMode exec_mode, epp_L2DataCacheSize l2d_size, epp_L1DataCachePFN pfvn);
};
